/* Name: main.c
 * Project: hid-data, example how to use HID for data transfer
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-11
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 692 2008-11-07 15:07:40Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>

#include "irmp.h"

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

#ifndef F_CPU
#error F_CPU unkown
#endif

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[90] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x0b, 0x01, 0x00, 0x00, 0xff,  // USAGE (Vendor Defined Page 1:Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x85, 0x01,                    //   REPORT_ID (1)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x06,                    //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0x85, 0x02,                    //   REPORT_ID (2)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0x85, 0x03,                    //   REPORT_ID (3)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x06,                    //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0x85, 0x04,                    //   REPORT_ID (4)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0x85, 0x05,                    //   REPORT_ID (5)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x06,                    //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};

/*
PROGMEM char usbHidReportDescriptor[60] = {    
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop) 
    0x09, 0x01,                    // USAGE (Vendor Usage 1) 
    0xa1, 0x01,                    // COLLECTION (Application) 
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0) 
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255) 
    0x75, 0x08,                    //   REPORT_SIZE (8) 
    0x85, 0x01,                    //   REPORT_ID (1) 
    0x95, 0x06,                    //   REPORT_COUNT (6) 
    0x09, 0x00,                    //   USAGE (Undefined) 
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf) 
    0x85, 0x02,                    //   REPORT_ID (2) 
    0x95, 0x01,                    //   REPORT_COUNT (1) 
    0x09, 0x00,                    //   USAGE (Undefined) 
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0x85, 0x03,                    //   REPORT_ID (3) 
    0x95, 0x06,                    //   REPORT_COUNT (6) 
    0x09, 0x00,                    //   USAGE (Undefined) 
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)	
    0x85, 0x04,                    //   REPORT_ID (4) 
    0x95, 0x01,                    //   REPORT_COUNT (1) 
    0x09, 0x00,                    //   USAGE (Undefined) 
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0x85, 0x05,                    //   REPORT_ID (5) 
    0x95, 0x06,                    //   REPORT_COUNT (6) 
    0x09, 0x00,                    //   USAGE (Undefined) 
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)		 	 
    0xc0                           // END_COLLECTION 
};

/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 128
 * opaque data bytes.
 */

/* The following variables store the status of the current data transfer */
static uchar    currentAddress   = 0;
static uchar    bytesRemaining   = 0;
static uchar	PowerOnEnabled   = 1;

static uchar    replyBuf[7];

// startup delay is needed for USB Init
// first run timer0, after a few seconds timer1 is starting
// and IR polling starts
static uchar	StartupDelay = 0xFF;

IRMP_DATA irmp_data;
IRMP_DATA last_received_irmp_data;

/* ------------------------------------------------------------------------- */
/* main functions for irmp
 */

//define output pin for power on switch:
#define SWITCH_PORT 		PORTC          /* PORTx - register for Switch output */
#define SWITCH_BIT  		PC5            /* bit for button input/output */
#define SWITCH_DDR  		DDRC           /* Switch data direction register */
#define LOCK_OUTPUT_TIME 	250     	   /* time to wait after a button press */


// register values from datasheet for ATMega8
#define OCIE1A      4
#define WGM12       3
#define CS10        0
#define UDRE0       5
#define TXEN0       3

void
timer_init (void)
{

	//Timer 0 init 
	TCNT0 = 0x00;   // Timer0 Counter to zero 
	//Timer Start
	TCCR0 = ( 1 << CS02 | 0 << CS01 | 1 << CS00 ) ; // Prescaler 1024 / Clock <- CPU 
	TIMSK = ( 1 << TOIE0 )   ;   // Timer0 ISR activate 

	OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                    // compare value: 1/10000 of CPU frequency
	//OCR1A   =  0x577;                                    // compare value: 1/10000 of CPU frequency
}

void 
init_io(void) 
{
uchar i, j;


    PORTB = 0xff;   /* activate all pull-ups */
    DDRB = 0;       /* all pins input */

    PORTC = 0xff;   /* activate all pull-ups */
	//PORTC ^= _BV(SWITCH_BIT);
    DDRC = 0; 		/* all pins input*/

    PORTD = 0xfa;   /* 1111 1010 bin: activate pull-ups except on USB lines */
    DDRD = 0x07;    /* 0000 0111 bin: all pins input except USB (-> USB reset) */

    /* set switch pin as digital output */
    SWITCH_DDR ^= _BV (SWITCH_BIT); //except power out pin 
	SWITCH_PORT ^= _BV(SWITCH_BIT);
	j = 0;
	while(--j){     /* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while(--i); /* delay >10ms for USB reset */
	}


    DDRD = 0x02;    /* 0000 0010 bin: remove USB reset condition */


        /* set switch pin as digital output */
        //SWITCH_DDR = _BV (SWITCH_BIT); 
 
        /* switch is OFF initially (set pin high) */         
        //SWITCH_PORT ^= _BV(SWITCH_BIT);
}

ISR(TIMER0_OVF_vect) /* Interrupt Routine */ 
{
   StartupDelay--;  

   TCNT0 = 0x00;             // Timer0 Counter to zero 
   
   if (StartupDelay == 0   )
   {
		TCCR0 = ( 0 << CS02 | 0 << CS01 | 0 << CS00 ) ; // Timer0 stop
		TCCR1B  = (1 << WGM12) | (1 << CS10);      		// switch CTC Mode on, set prescaler to 1
		TIMSK = ( 0 << TOIE0 | 1 << OCIE1A )   ;   		// Timer0 ISR deaktivate, Timer1A ISR activate 
   }
}  

 

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// Timer 1 output compare A interrupt service routine

ISR(TIMER1_COMPA_vect)
{

	//	usbDisableAllRequests();
  irmp_ISR();                                                           // call irmp ISR
  //	usbEnableAllRequests();
  // call other timer interrupt routines...
}

/* ------------------------------------------------------------------------- */

/* usbFunctionRead() is called when the host requests a chunk of data from
 * the device. For more information see the documentation in usbdrv/usbdrv.h.
 */
/*
uchar   usbFunctionRead(uchar *data, uchar len)
{
    if(len > bytesRemaining)
        len = bytesRemaining;
    //eeprom_read_block(data, (uchar *)0 + currentAddress, len);

	//copy data by ignoring the first byte (reportnumber)

                data[0] = irmp_data.protocol; 
                data[1] = irmp_data.address & 0xFF;
                data[2] = irmp_data.address >> 8; 
                data[3] = irmp_data.command & 0xFF; 
                data[4] = irmp_data.command >> 8; 
                data[5] = irmp_data.flags; 

    currentAddress += len;
    bytesRemaining -= len;
    return len;
}

/* ------------------------------------------------------------------------- */

/* usbFunctionWrite() is called when the host sends a chunk of data to the
 * device. For more information see the documentation in usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
static uchar i;
static uchar    Buf[8]; 

    if(bytesRemaining == 0)
        return 1;               /* end of transfer */
    if(len > bytesRemaining)
        len = bytesRemaining;

	//copy data by ignoring the first byte (reportnumber)
	for (i=0; i<len; i++){
		Buf[i] = data[i+1];
	};

    eeprom_write_block(&Buf, (uchar *)0 + currentAddress, len); 
    currentAddress += len;
    bytesRemaining -= len;
    return bytesRemaining == 0; /* return 1 if this was the last chunk */
}

/* ------------------------------------------------------------------------- */

uchar   usbFunctionSetup(uchar data[8]) 
{ 

usbRequest_t    *rq = (void *)data; 

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    // HID class request 
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  // wValue: ReportType (highbyte), ReportID (lowbyte)


		usbMsgPtr = replyBuf;

            if(rq->wValue.bytes[0] == 1){   /* ReportID 1 */ 

				if ( last_received_irmp_data.protocol != 0)
				{
	                replyBuf[0] = rq->wValue.bytes[0];
				
	                replyBuf[1] = last_received_irmp_data.protocol; 
	                replyBuf[2] = last_received_irmp_data.address & 0xFF;
	                replyBuf[3] = last_received_irmp_data.address >> 8; 
	                replyBuf[4] = last_received_irmp_data.command & 0xFF; 
	                replyBuf[5] = last_received_irmp_data.command >> 8; 
	                replyBuf[6] = last_received_irmp_data.flags; 
				
					//clear irmp_data for next polling
					//clear irmp_data
					last_received_irmp_data.protocol = 0;
					last_received_irmp_data.address = 0;
					last_received_irmp_data.command = 0;
					last_received_irmp_data.flags = 0;
													 
                	return 7;
				}
				else
				{
					//return no new code
					return 0;
				}

				
            }else if(rq->wValue.bytes[0] == 2){ /* ReportID 2 */ 
                replyBuf[0] = rq->wValue.bytes[0];
				eeprom_read_block(&replyBuf[1], 0, 1); 
                return 2;  
            }else if(rq->wValue.bytes[0] == 3){ /* ReportID 3 */ 
                replyBuf[0] = rq->wValue.bytes[0];
				eeprom_read_block(&replyBuf[1], 1, 6);  
                return 7;  
            }
			 
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){ 
            if(rq->wValue.bytes[0] == 4){   /* ReportID 4 */
				//store PowerOnEnabled in EEProm:
	            bytesRemaining = 1;
	            currentAddress = 0;
	            return USB_NO_MSG;  // use usbFunctionWrite() to receive data from host	
            }else if(rq->wValue.bytes[0] == 5){  /* ReportID 5 */
				//store IR Code in EEProm
	            bytesRemaining = 6;
	            currentAddress = 1;
	            return USB_NO_MSG;  // use usbFunctionWrite() to receive data from host	
            } 
        } 
    }else{ 
        // ignore vendor type requests, we don't use any
    } 
    return 0; 
} 

/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;
IRMP_DATA trained_irmp_data;

    wdt_enable(WDTO_2S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    odDebugInit();
    usbInit();

	init_io();

	//clear irmp_data
	irmp_data.protocol = 0;
	irmp_data.address = 0;
	irmp_data.command = 0;
	irmp_data.flags = 0;

	//clear irmp_data
	last_received_irmp_data.protocol = 0;
	last_received_irmp_data.address = 0;
	last_received_irmp_data.command = 0;
	last_received_irmp_data.flags = 0;


	irmp_init();                                                              // initialize irmp code
	timer_init();                                                             // initialize timer

    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();

	//set PowerOn Enabled to default:
	eeprom_write_block(&PowerOnEnabled, 0, 1);

    for(;;){                /* main event loop */
        wdt_reset();

		//check if new IR code is aviable
	    if (irmp_get_data (&irmp_data))
	    {
	        // ir signal decoded, do something here...
	        // irmp_data.protocol is the protocol, see irmp.h
	        // irmp_data.address is the address/manufacturer code of ir sender
	        // irmp_data.command is the command code
			last_received_irmp_data.protocol = irmp_data.protocol;
			last_received_irmp_data.address = irmp_data.address;
			last_received_irmp_data.command = irmp_data.command;
			last_received_irmp_data.flags = irmp_data.flags;

			
			eeprom_read_block(&trained_irmp_data, 1, 6); 

			//check if a code is assigned:
			if ((trained_irmp_data.protocol != 0xFF) & (trained_irmp_data.protocol != 0x00))
			{

				eeprom_read_block(&PowerOnEnabled, 0, 1);
				//if power on is enabled check it
				if (PowerOnEnabled == 1)
				{				 

					if ((last_received_irmp_data.protocol == trained_irmp_data.protocol) &
						(last_received_irmp_data.address == trained_irmp_data.address) &
						(last_received_irmp_data.command == trained_irmp_data.command))
					{
						//clear irmp_data
						last_received_irmp_data.protocol = 0;
						last_received_irmp_data.address = 0;
						last_received_irmp_data.command = 0;
						last_received_irmp_data.flags = 0;
						//siwtch on output port pin
						SWITCH_PORT ^= _BV(SWITCH_BIT);
						_delay_ms(LOCK_OUTPUT_TIME);
						SWITCH_PORT ^= _BV(SWITCH_BIT);
					}
				}
			}
			else
			{
				//no code trained, save it:
				eeprom_write_block(&last_received_irmp_data, 1, 6);
				//update trained_irmp_data
				eeprom_read_block(&trained_irmp_data, 1, 6); 
			}
	    }
		
		//if ( !usbAllRequestsAreDisabled() );
		//{
        	usbPoll();
		//}
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
